@@ -31,7 +31,8 @@ import threading
 import os
 import pandas as pd
 from time import sleep
-from qff.frame.context import context, strategy, RUNSTATUS, RUNTYPE, run_strategy_funcs
+from qff.frame.context import context, strategy, run_strategy_funcs
+from qff.frame.const import RUN_TYPE, RUN_STATUS
 from qff.frame.backup import save_context
 from qff.frame.order import order_broker
 from qff.frame.settle import settle_by_day, profit_analyse
@@ -42,26 +43,19 @@ from qff.tools.logs import log
 from qff.tools.local import cache_path
 
 
-def sim_trade_run(resume=False):
+def sim_trade_run():
     """
     实盘模拟框架运行函数,执行该函数将运行策略实盘模拟
 
-    :param resume: 是否为恢复以前中断的策略，默认全新开始,该参数在__main__.py文件中使用
     :return: None
     """
-    context.run_type = RUNTYPE.SIM_TRADE
+    context.run_type = RUN_TYPE.SIM_TRADE
 
-    if context.status == RUNSTATUS.RUNNING:
+    if context.status == RUN_STATUS.RUNNING:
         log.error('实盘模拟框架函数已运行！')
         return
     else:
-        context.status = RUNSTATUS.RUNNING
-
-    if resume:
-        if strategy.process_initialize is not None:
-            strategy.process_initialize()
-    else:
-        strategy.initialize()
+        context.status = RUN_STATUS.RUNNING
 
     # 恢复运行时不能设置
     if context.bm_start is None:
@@ -79,7 +73,7 @@ def sim_trade_run(resume=False):
 
 
 def _sim_trade_run():
-    while context.status == RUNSTATUS.RUNNING:
+    while context.status == RUN_STATUS.RUNNING:
         _time = pd.Timestamp.now()
         stime = _time.strftime('%Y-%m-%d %H:%M:%S')
         if is_trade_day(stime[0:10]):
@@ -93,26 +87,28 @@ def _sim_trade_run():
                     run_strategy_funcs(strategy.before_trading_start)
                 sleep((_time.ceil(freq="1min") - pd.Timestamp.now()).total_seconds())
             elif '09:30' <= stime[11:16] <= '11:30' or '13:00' <= stime[11:16] <= '15:00':
-                # # 固定时间点的策略函数
-                # if stime[11:] in strategy.run_daily.keys():
-                #     strategy.run_daily[stime[11:]]()
                 # 按策略频率运行的策略函数
-                if strategy.handle_data is not None:
-                    run_strategy_funcs(strategy.handle_data)
+                if context.run_freq == 'day':
+                    if stime[11:16] == '09:30' and strategy.handle_data is not None:
+                        run_strategy_funcs(strategy.handle_data)
+                        sleep((_time.ceil(freq="1min") - pd.Timestamp.now()).total_seconds())
+                else:
+                    if strategy.handle_data is not None:
+                        run_strategy_funcs(strategy.handle_data)
 
                 # 订单撮合 order_broker
                 order_broker()
 
-                _freq = "1min" if context.run_freq == 'min' else '3s'     # 3s是为了tick频率运行
+                _freq = "3s" if context.run_freq == 'tick' else '1min'     # 3s是为了tick频率运行
                 _t = pd.Timestamp.now()
                 sleep((_t.ceil(freq=_freq) - _t).total_seconds())         # 考虑前面运行超过_freq，造成sleep负数
                 # sleep((pd.Timestamp.now().ceil(freq='3s') - pd.Timestamp.now()).total_seconds())
 
             elif stime[11:16] == '15:30':
+                settle_by_day()
                 if strategy.after_trading_end is not None:
                     run_strategy_funcs(strategy.after_trading_end)
 
-                settle_by_day()
                 profit_analyse()
                 save_context()
                 log.info("##################### 一天结束 ######################")
@@ -123,7 +119,7 @@ def _sim_trade_run():
                 sleep((_time.ceil(freq="1min") - pd.Timestamp.now()).total_seconds())
         else:
             sleep(60)
-    if context.status == RUNSTATUS.PAUSED:
+    if context.status == RUN_STATUS.PAUSED:
         log.warning("回测运行暂停，保存过程数据...!")
         default_name = context.strategy_name+'_bt.pkl'
         if ' ' in default_name:
@@ -135,5 +131,5 @@ def _sim_trade_run():
         print(f"策略备份文件：{backup_file}")
         save_context(backup_file)
         save_context()
-    elif context.status == RUNSTATUS.CANCELED:
+    elif context.status == RUN_STATUS.CANCELED:
         log.warning("回测执行取消...!")
