@@ -28,7 +28,7 @@
 import pandas as pd
 from datetime import datetime
 from qff.price.query import get_st_stock, get_paused_stock, get_price, get_stock_list
-from qff.price.report import get_history_fundamentals
+from qff.price.finance import get_history_fundamentals
 from qff.tools.date import  get_real_trade_date
 
 
@@ -112,7 +112,7 @@ def select_npgr_stock(npgr, date=None, count=1):
         date = datetime.now().strftime('%Y-%m-%d')
         date = get_real_trade_date(date, -1)
 
-    data = get_history_fundamentals(code=None, fields=['184'], watch_date=date, count=count)
+    data = get_history_fundamentals(code=None, fields=['f184'], watch_date=date, count=count)
     data = data.rename(columns={'184' : 'npgr'}).drop(['report_date', 'pub_date'], axis=1)
     # data = data[data['npgr'] >= npgr]
     df = data.groupby(['code']).apply(lambda x: (x['npgr'] >= npgr).all())
